@@ -7,7 +7,7 @@
  *
  * perdition
  * Mail retrieval proxy server
- * Copyright (C) 1999-2004  Horms
+ * Copyright (C) 1999-2005  Horms
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -142,9 +142,14 @@ options_t opt;
   OPT_MODIFY_USERNAME(opt.strip_domain, opt.mask, MASK_STRIP_DOMAIN, f, \
     "strip_domain");
 
-#define OPT_ADD_DOMAIN \
-  OPT_MODIFY_USERNAME(opt.add_domain, opt.mask, MASK_ADD_DOMAIN, f, \
-    "add_domain");
+#define OPT_ADD_DOMAIN							\
+	if (vanessa_socket_str_is_digit(optarg_copy)) {			\
+		opt.add_domain_strip_depth = atoi(optarg_copy);		\
+	}								\
+	else {								\
+		OPT_MODIFY_USERNAME(opt.add_domain, opt.mask, 		\
+				MASK_ADD_DOMAIN, f, "add_domain");	\
+	}
 
 #define OPT_LOWER_CASE \
   OPT_MODIFY_USERNAME(opt.lower_case, opt.mask, MASK_LOWER_CASE, f, \
@@ -263,6 +268,7 @@ int options(int argc, char **argv, flag_t f){
     {"client_server_specification", 'c',  POPT_ARG_NONE,   NULL, 'c'},
     {"domain_delimiter",            'D',  POPT_ARG_STRING, NULL, 'D'},
     {"debug",                       'd',  POPT_ARG_NONE,   NULL, 'd'},
+    {"explicit_domain",             'e',  POPT_ARG_STRING, NULL, 'e'},
     {"log_facility",                'F',  POPT_ARG_STRING, NULL, 'F'},
     {"config_file",                 'f',  POPT_ARG_STRING, NULL, 'f'},
     {"group",                       'g',  POPT_ARG_STRING, NULL, 'g'},
@@ -340,6 +346,7 @@ int options(int argc, char **argv, flag_t f){
   /* i is used as a dummy variable */
   if(f&OPT_SET_DEFAULT){
     opt_i(opt.add_domain,      DEFAULT_ADD_DOMAIN,          i, 0, OPT_NOT_SET);
+    opt.add_domain_strip_depth = DEFAULT_ADD_DOMAIN_STRIP_DEPTH;
 #ifdef WITH_PAM_SUPPORT
     opt_i(opt.authenticate_in, DEFAULT_AUTHENTICATE_IN,     i, 0, OPT_NOT_SET);
 #endif /* WITH_PAM_SUPPORT */
@@ -372,7 +379,6 @@ int options(int argc, char **argv, flag_t f){
     opt_i(opt.no_lookup,       DEFAULT_NO_LOOKUP,           i, 0, OPT_NOT_SET);
     opt_i(opt.login_disabled,  DEFAULT_LOGIN_DISABLED,      i, 0, OPT_NOT_SET);
     opt_i(opt.lower_case,      DEFAULT_LOWER_CASE,          i, 0, OPT_NOT_SET);
-    opt_i(opt.add_domain,      DEFAULT_LOWER_CASE,          i, 0, OPT_NOT_SET);
     opt_i(opt.server_resp_line,DEFAULT_SERVER_RESP_LINE,    i, 0, OPT_NOT_SET);
     opt_i(opt.strip_domain,    DEFAULT_STRIP_DOMAIN,        i, 0, OPT_NOT_SET);
     opt_i(opt.timeout,         DEFAULT_TIMEOUT,             i, 0, OPT_NOT_SET);
@@ -390,6 +396,7 @@ int options(int argc, char **argv, flag_t f){
       opt_p(opt.config_file,   filename,                    i, 0, OPT_NOT_SET);
     }
     opt_p(opt.domain_delimiter,DEFAULT_DOMAIN_DELIMITER,    i, 0, OPT_NOT_SET);
+    opt_p(opt.explicit_domain, NULL,                        i, 0, OPT_NOT_SET);
     opt_p(opt.group,           DEFAULT_GROUP,               i, 0, OPT_NOT_SET);
     opt_p(opt.listen_port,     PERDITION_PROTOCOL_DEPENDANT,i, 0, OPT_NOT_SET);
     opt_p(opt.map_library,     DEFAULT_MAP_LIB,             i, 0, OPT_NOT_SET);
@@ -508,6 +515,9 @@ int options(int argc, char **argv, flag_t f){
       case 'd':
         opt_i(opt.debug,1,opt.mask,MASK_DEBUG,f);
         break;
+      case 'e':
+	opt_p(opt.explicit_domain,optarg,opt.mask2,MASK2_EXPLICIT_DOMAIN,f);
+	break;
       case 'f':
         if(!(f&OPT_FILE)){
           opt_p(opt.config_file,optarg,opt.mask,MASK_CONFIG_FILE,f);
@@ -920,6 +930,12 @@ int log_options_str(char *str, size_t n){
   }
 
   LOG_OPTIONS_BUILD_USERNAME_MODIFIER(opt.add_domain, add_domain);
+  if (*add_domain) {
+    snprintf(add_domain + strlen(add_domain), 
+		    39 - strlen(add_domain), ",%d", 
+		    opt.add_domain_strip_depth);
+    add_domain[39] = '\0';
+  }
   LOG_OPTIONS_BUILD_USERNAME_MODIFIER(opt.lower_case, lower_case);
   LOG_OPTIONS_BUILD_USERNAME_MODIFIER(opt.strip_domain, strip_domain);
   
@@ -982,6 +998,7 @@ int log_options_str(char *str, size_t n){
     "connect_relog=%d, "
     "debug=%s, "
     "domain_delimiter=\"%s\", "
+    "explicit_domain=\"%s\", "
     "group=\"%s\", "
     "inetd_mode=%s, "
     "listen_port=\"%s\", "
@@ -1021,9 +1038,9 @@ int log_options_str(char *str, size_t n){
     "ssl_outgoing_ciphers=\"%s\", "
     "ssl_no_cert_verify=\"%s\", "
     "ssl_no_cn_verify=\"%s\", "
-    "(ssl_mask=0x%x) "
+    "(ssl_mask=0x%08x) "
 #endif /* WITH_SSL_SUPPORT */
-    "(mask=0x%x)\n",
+    "(mask=0x%08x %08x)\n",
     OPT_STR(add_domain),
 #ifdef WITH_PAM_SUPPORT
     BIN_OPT_STR(opt.authenticate_in),
@@ -1037,6 +1054,7 @@ int log_options_str(char *str, size_t n){
     opt.connect_relog,
     BIN_OPT_STR(opt.debug),
     OPT_STR(opt.domain_delimiter),
+    OPT_STR(opt.explicit_domain),
     OPT_STR(opt.group),
     BIN_OPT_STR(opt.inetd_mode),
     OPT_STR(opt.listen_port),
@@ -1078,7 +1096,7 @@ int log_options_str(char *str, size_t n){
     BIN_OPT_STR(opt.ssl_no_cn_verify),
     opt.ssl_mask,
 #endif /* WITH_SSL_SUPPORT */
-    opt.mask
+    opt.mask, opt.mask2
   );
   *(str+n-1) = '\0';
 
@@ -1163,7 +1181,7 @@ void usage(int exit_status){
     "Usage: perdition [options]\n"
     "\n"
     "Options:\n"
-    " -A|--add_domain STATE[,STATE...]:\n"
+    " -A|--add_domain STATE[,STATE...][,STRIP_DEPTH]:\n"
     "    Appends a domain to the USER based on the IP address connected to\n"
     "    in given state(s). (default \"\")\n"
 #ifdef WITH_PAM_SUPPORT
@@ -1188,6 +1206,9 @@ void usage(int exit_status){
     "    Delimiter between username and domain. (default \"%s\")\n"
     " -d|--debug:\n"
     "    Turn on verbose debugging.\n"
+    " -e|--explicit_domain STRING:\n"
+    "    With -A, use STRING as the default domain rather than deriving\n"
+    "    from the IP address connected to.\n"
     " -F|--log_facility FACILITY:\n"
     "    Facility to log to. (default \"%s\")\n"
     " -f|--config_file FILENAME:\n"
