@@ -7,7 +7,7 @@
  *
  * perdition
  * Mail retrieval proxy server
- * Copyright (C) 1999-2004  Horms
+ * Copyright (C) 1999-2005  Horms
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -100,6 +100,7 @@
 
 
 #define DEFAULT_ADD_DOMAIN                   STATE_NONE
+#define DEFAULT_ADD_DOMAIN_STRIP_DEPTH       1
 #ifdef WITH_PAM_SUPPORT
 #define DEFAULT_AUTHENTICATE_IN              0
 #endif /* WITH_PAM_SUPPORT */
@@ -167,6 +168,7 @@
 
 typedef struct {
   int             add_domain;
+  unsigned int    add_domain_strip_depth;
 #ifdef WITH_PAM_SUPPORT
   int             authenticate_in;
 #endif /* WITH_PAM_SUPPORT */
@@ -181,6 +183,7 @@ typedef struct {
   int             debug;
   char            *domain_delimiter;
   int             domain_delimiter_length;
+  char            *explicit_domain;
   char            *group;
   int             inetd_mode;
   char            *pop_capability;
@@ -263,6 +266,7 @@ typedef struct {
 
 #define MASK2_OK_LINE                    (flag_t) 0x00000001
 #define MASK2_PID_FILE                   (flag_t) 0x00000002
+#define MASK2_EXPLICIT_DOMAIN            (flag_t) 0x00000004
 
 #ifdef WITH_SSL_SUPPORT
 /* options_t.ssl_mask entries */
