@@ -6,7 +6,7 @@
  *
  * perdition
  * Mail retrieval proxy server
- * Copyright (C) 1999-2004  Horms
+ * Copyright (C) 1999-2005  Horms
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -70,7 +70,8 @@ int quit(io_t * io, const protocol_t * protocol, token_t * tag)
 	token_assign(t, (PERDITION_USTRING) protocol->type[PROTOCOL_OK],
 		     strlen(protocol->type[PROTOCOL_OK]), TOKEN_DONT_CARE);
 
-	if ((protocol->out_response(io, tag, t, &q, NULL, NULL)) < 0) {
+	if ((protocol->out_response(io, NULL, tag, t, &q, NULL, 
+					NULL)) < 0) {
 		VANESSA_LOGGER_DEBUG("out_response");
 		goto leave;
 	}
